<?php

/**
 * @Module Name: 1Pay Payments | Magento
 * Plugin URI: http://1pay.in
 * Description: Extends Magento with 1Pay services
 * Version: 1.0.3
 * @author: Soumil Gada, Manisha Gupta
 * Author EMAIL: soumil.gada@mobilewaretech.com
 * @copyright: @2018 Mobilewaretech. All rights reserved
 */

namespace MwtUM\Onepay\Controller\Ipn;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Action\Action as AppAction;

class CallbackMwtum extends AppAction {

    /**
     * @var \Citrus\Icp\Model\PaymentMethod
     */
    protected $_paymentMethod;

    /**
     * @var \Magento\Sales\Model\Order
     */
    protected $_order;

    /**
     * @var \Magento\Sales\Model\OrderFactory
     */
    protected $_orderFactory;

    /**
     * @var Magento\Sales\Model\Order\Email\Sender\OrderSender
     */
    protected $_orderSender;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;
    protected $request;

    /**
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Magento\Sales\Model\OrderFactory $orderFactory
     * @param \Citrus\Icp\Model\PaymentMethod $paymentMethod
     * @param Magento\Sales\Model\Order\Email\Sender\OrderSender $orderSender
     * @param  \Psr\Log\LoggerInterface $logger
     */
    public function __construct(
    \Magento\Framework\App\Action\Context $context, \Magento\Framework\App\Request\Http $request, \Magento\Sales\Model\OrderFactory $orderFactory, \MwtUM\Onepay\Model\PaymentMethod $paymentMethod, \Magento\Sales\Model\Order\Email\Sender\OrderSender $orderSender, \Psr\Log\LoggerInterface $logger
    ) {
        $this->_paymentMethod = $paymentMethod;
        $this->_orderFactory = $orderFactory;
        $this->_client = $this->_paymentMethod->getClient();
        $this->_orderSender = $orderSender;
        $this->_logger = $logger;
        $this->request = $request;
        parent::__construct($context);
    }

    /**
     * Handle POST request to MwtUMoney callback endpoint.
     */
    public function execute() {

        try {
            if (!empty($this->request->getPostValue())) {
                $this->_success();
                $this->paymentAction();
            } else {
                $this->_logger->addError("MwtUMoney: no post back data received in callback");
                $this->_redirect('checkout/onepage/failure');
                return $this->_failure();
            }
        } catch (Exception $e) {
            $this->_logger->addError("MwtUMoney: error processing callback");
            $this->_logger->addError($e->getMessage());
            $this->_redirect('checkout/onepage/failure');
            return $this->_failure();
        }

        $this->_logger->addInfo("MwtUMoney Transaction END from MwtUMoney");
    }

    function checkTransactionStatus($data) {

        $respData = $data['respData'];
        $url = $data['url'];
        $curl = curl_init();


        $respDecrypt = $this->get_decrypt($respData);
        $verificationResult = json_decode($respDecrypt);

        $merchantid = $verificationResult->merchant_id;
        $txnid = $verificationResult->txn_id;


        $myvars = "merchantId=" . $merchantid . "&txnId=" . $txnid . "";

        curl_setopt_array($curl, array(
            CURLOPT_PORT => "8080",
            CURLOPT_URL => $url . "onepayVAS/getTxnDetails",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_POSTFIELDS => $myvars,
                //   CURLOPT_HTTPHEADER => array(
                //     "Content-Type: application/json",
                //   ),
        ));
        $response = curl_exec($curl);

        return json_decode($response);
    }

    function get_string_between($string, $start, $end) {
        $string = ' ' . $string;
        $ini = strpos($string, $start);
        if ($ini == 0)
            return '';
        $ini += strlen($start);
        $len = strpos($string, $end, $ini) - $ini;
        return substr($string, $ini, $len);
    }

    protected function paymentAction() {
        $merchantid = $this->_paymentMethod->getConfigData('merchantId');
        $apikey = $this->_paymentMethod->getConfigData('apiKey');
        $environment = $this->_paymentMethod->getConfigData('environment');

        $action = "https://pay.1paypg.in/";
        if ($environment == 'sandbox')
            $action = "http://139.59.1.254:8080/";
        if ($this->getRequest()->isPost()) {
            $postdata = $this->getRequest()->getPostValue();
            if (isset($postdata['respData'])) {
                $respData = $postdata['respData'];

                $checkTrans = $this->checkTransactionStatus(array('respData' => $postdata['respData'], 'url' => $action));
                $checkTransData['parsedtxnid'] = $checkTrans->txn_id; //this->get_string_between($checkTrans, '<txn_id>', '</txn_id>');
                $checkTransData['parsedmerchantid'] = $checkTrans->merchant_id; //$this->get_string_between($checkTrans, '<merchant_id>', '</merchant_id>');
                $checkTransData['parsedtxnamount'] = $checkTrans->txn_amount; // $this->get_string_between($checkTrans, '<txn_amount>', '</txn_amount>');
                $checkTransData['parsedtxnstatus'] = $checkTrans->trans_status; // $this->get_string_between($checkTrans, '<trans_status>', '</trans_status>');
                $checkTransData['pg_ref_id'] = $checkTrans->pg_ref_id; //$this->get_string_between($checkTrans, '<pg_ref_id>', '</pg_ref_id>');
                $ordid = explode('_', $checkTrans->txn_id);
                $ordid = $ordid[0];


                $this->_loadOrder($ordid);
                $amount = $checkTrans->txn_amount;
                $email = $checkTrans->cust_email_id;
                $message = '';
                $message .= 'orderId / Transaction ID: ' . $ordid . "\n";


                if ($checkTrans->trans_status === "Ok") {
                    if ($checkTrans->txn_id !== "NA") {
                        $message .= 'payid : ' . $checkTrans->bank_ref_id . "\n";
                        $message .= "Verified Transaction" . "\n";
                        if ($checkTrans->trans_status == 'Ok') {
                            $message1 = "Transaction Successful";
                            $this->_registerPaymentCapture($ordid, $amount, $message);
                            $this->_logger->addInfo("Mwtum Response Order success.." . $message1);
                            $redirectUrl = $this->_paymentMethod->getSuccessUrl();
                            $this->_redirect($redirectUrl);
                        } else if ($checkTrans->trans_status == 'To') {
                            $message1 = "Sorry!!Your Transaction is Timed Out";
                            $historymessage = $message1;
                            $this->_createMwtUMComment($historymessage);
                            $this->_order->cancel()->save();
                            $this->messageManager->addError("<strong>Error:</strong> $message1 <br/>");

                            $redirectUrl = $this->_paymentMethod->getCancelUrl();
                            $this->_redirect($redirectUrl);
                        } else {
                            $message1 = "Transaction Failed";
                            $historymessage = $message1;
                            $this->_createMwtUMComment($historymessage);
                            $this->_order->cancel()->save();
                            $this->_logger->addInfo("Mwtum Response Order cancelled ..");
                            $this->messageManager->addError("<strong>Error:</strong> $message1 <br/>");
                            $redirectUrl = $this->_paymentMethod->getCancelUrl();
                            $this->_redirect($redirectUrl);
                        }
                    } else {
                        $vmessage = "Transaction Failed";
                        $historymessage = $message1;

                        $this->_createMwtUMComment($historymessage);
                        $this->_order->cancel()->save();
                        $this->_logger->addInfo("Mwtum Response Order cancelled ..");
                        $this->messageManager->addError("<strong>Error:</strong> $vmessage <br/>");
                        $redirectUrl = $this->_paymentMethod->getCancelUrl();
                        $this->_redirect($redirectUrl);
                    }
                } else {
                    $message .= "\n Your Order Failed. Try Again Later";
                    $historymessage = $message;
                    $this->_createMwtUMComment($historymessage);
                    $this->_order->cancel()->save();
                    $this->messageManager->addError("<strong>Error:</strong> $message <br/>");
                    $redirectUrl = $this->_paymentMethod->getCancelUrl();
                    $this->_redirect($redirectUrl);
                }
            }
        }
    }

    protected function _registerPaymentCapture($transactionId, $amount, $message) {
        $payment = $this->_order->getPayment();


        $payment->setTransactionId($transactionId)
                ->setPreparedMessage($this->_createMwtUMComment($message))
                ->setShouldCloseParentTransaction(true)
                ->setIsTransactionClosed(0)
                ->setAdditionalInformation(['onepay', 'Mwtumoney'])
                ->registerCaptureNotification(
                        $amount, true
        );
        $this->_order->save();

        $invoice = $payment->getCreatedInvoice();
        if ($invoice && !$this->_order->getEmailSent()) {
            $this->_orderSender->send($this->_order);
            $this->_order->addStatusHistoryComment(
                    __('You notified customer about invoice #%1.', $invoice->getIncrementId())
            )->setIsCustomerNotified(
                    true
            )->save();
        }
    }

    protected function _loadOrder($order_id) {
        $this->_order = $this->_orderFactory->create()->loadByIncrementId($order_id);

        if (!$this->_order && $this->_order->getId()) {
            throw new Exception('Could not find Magento order with id $order_id');
        }
    }

    protected function _success() {
        $this->getResponse()
                ->setStatusHeader(200);
    }

    protected function _failure() {
        $this->getResponse()
                ->setStatusHeader(400);
    }

    /**
     * Returns the generated comment or order status history object.
     *
     * @return string|\Magento\Sales\Model\Order\Status\History
     */
    protected function _createMwtUMComment($message = '') {
        if ($message != '') {
            $message = $this->_order->addStatusHistoryComment($message);
            $message->setIsCustomerNotified(null);
        }

        return $message;
    }

}
